==> animal-2.2.0.1.tar.gz: animal-2.2.0.1/bin/utils.c <==
/*
 *  Urgent bits first
 */

static char *utils_c_rcs_id = "$Header: /diskb/home/alexis/dev/supported/support/lib/RCS/utils.c,v 1.12 1999/02/20 16:29:51 alexis Exp $";

/*
 *  System includes - with reasons
 */

#include <stdio.h>				/* for NULL and *printf*() */
#include <stdarg.h>				/* for va_*() */
#include <string.h>				/* for strrchr(), strerror() */
#include <sys/types.h>                          /* for umask() */
#include <sys/stat.h>                   	/* for stat(), umask() */
#include <unistd.h>                    		/* for stat() */
#include <errno.h>				/* for errno */
#include <stdlib.h>				/* for exit() */
#ifndef MSDOS
#include <syslog.h>				/* for syslog(), LOG_*, etc */
#endif

/*
 *  Local includes with reasons			
 */

#include "utils.h" 				/* for pp symbols */

/*
 *  Private structure definitions
 */

struct delonexitrec {
    char *name;
    struct delonexitrec *next;
};

/*
 *  Private variables 
 */

static int state_syslog_opened = FALSE;
static struct delonexitrec *delonexitlist = NULL;

/*
 *  Global variables belonging to this file
 */

char *progname;
int i_am_a_daemon = FALSE;
unsigned long debug_level = DBG_WARNING|DBG_ERROR;

/*
 *  Functions start here
 */

char *mybasename(
char *cmdcp,
char *extnp)
{
    char *bncp;
    static char basenamebuf[64];

    bncp = ((bncp=strrchr(cmdcp, '/')) == NULL) ? cmdcp : bncp+1;

    if (extnp == NULL || *extnp == '\0' || strcmp(bncp+strlen(bncp)-strlen(extnp), extnp))
        return(bncp);
    else {
        strncpy2(basenamebuf, bncp, strlen(bncp)-strlen(extnp));
        return(basenamebuf);
    }
}

void internal(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    va_start(args, fmt);
    sprintf(msgbuf, "INTERNAL ERROR: ");
    vsprintf(msgbuf+strlen(msgbuf), fmt, args);
    va_end(args);

    if (!i_am_a_daemon)
        fprintf(stderr, "%s: %s\n", progname, msgbuf);
    else {
#ifndef MSDOS
        if (!state_syslog_opened) {
            openlog(progname, LOG_CONS|LOG_PID, FACILITY);
            state_syslog_opened = TRUE;
        }
        syslog(LOG_EMERG, msgbuf);
#else
        fprintf(stderr, "%s: %s\n", progname, msgbuf);
#endif
    }
    exit(255);
}

void error(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_ERROR) {
        va_start(args, fmt);
        sprintf(msgbuf, "ERROR: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_ERR, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
    exit(EXIT_FAILURE);
}

void warning(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_WARNING) {
        va_start(args, fmt);
        sprintf(msgbuf, "WARNING: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);
    
        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_WARNING, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

void info(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_INFO) {
        va_start(args, fmt);
        sprintf(msgbuf, "INFO: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_INFO, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

void debug(
int level,
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (level & debug_level) {
        va_start(args, fmt);
        sprintf(msgbuf, "DEBUG[%04x]: ", level);
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_DEBUG, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

/*
 *  A string copy function. It can cope with 'left-shift' overlaps, but
 *  not 'right-shifts'.
 */

char *strmove(
char *dst,
char *src)
{
    while (*dst++ = *src++)
        ;
}

pid_t my_lock(
char *lockwhat,
int (*gen_lock_file_name_fnc)(char *, char *))
{
    char lock_file[128];
    char tmp_lock_file[128];
    pid_t locking_pid;
    FILE *fp;
    struct stat statbuf;
    char procbuf[16];
    mode_t old_umask;

    debug(DBG_FUNCS, "my_lock: sof");

    gen_lock_file_name_fnc(lock_file, lockwhat);
    debug(DBG_LOCKS, "my_lock: lockfile=%s", lock_file);
    sprintf(tmp_lock_file, "%s.%d", lock_file, getpid());
    debug(DBG_LOCKS, "my_lock: tmplockfile=%s", tmp_lock_file);

    /* create temporary lock_file */
    debug(DBG_LOCKS, "my_lock: creating temporary lock");
    umask((old_umask = umask(0)) | 0444);
    delonexit(tmp_lock_file);
    if ((fp=fopen(tmp_lock_file, "w")) == (FILE *) NULL) {
	umask(old_umask);
        debug(DBG_LOCKS, "my_lock: temporary lock creation failed (errno=%d)", errno);
        dontdelonexit(tmp_lock_file);
        return(-1); 
    }
    umask(old_umask);
    fprintf(fp, "%d\n", getpid());
    fclose(fp);

    /* slide lock into place */
    debug(DBG_LOCKS, "my_lock: hard-linking temporary lock to real lock");
    delonexit(lock_file);
    if (link(tmp_lock_file, lock_file) == 0) {
        debug(DBG_LOCKS, "my_lock: hard-linking succeeded");
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(0);
    }
    dontdelonexit(lock_file);
    debug(DBG_LOCKS, "my_lock: hard-linking failed");

    /* if blocked examine the existing lock */
    debug(DBG_LOCKS, "my_lock: examining existing lock");
    if ((fp=fopen(lock_file, "r")) == (FILE *) NULL) {
        debug(DBG_LOCKS, "my_lock: lock file open for reading failed");
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(-1);
    }
    debug(DBG_LOCKS, "my_lock: lock file read failed");
    if (fscanf(fp, "%d", (int *) &locking_pid) != 1) {
        fclose(fp);
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(-1);
    }
    fclose(fp);

    debug(DBG_LOCKS, "my_lock: found a pid in lock file, checking if running");
    sprintf(procbuf, "/proc/%d", (int) locking_pid);
    if (stat(procbuf, &statbuf) == 0) {
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(locking_pid);
    }

    info("stale lock %s removed (pid=%d)", lock_file, (int) locking_pid);
    unlink(lock_file);

    /* slide lock into place */
    debug(DBG_LOCKS, "my_lock: hard-linking temporary lock to real lock (2nd attmept");
    delonexit(lock_file);
    if (link(tmp_lock_file, lock_file) == 0) {
        unlink(tmp_lock_file);
        return(0);
    }

    unlink(tmp_lock_file);
    dontdelonexit(tmp_lock_file);
    return(-1);
}

void my_unlock(
char *lockwhat,
int (*gen_lock_file_name_fnc)(char *, char *))
{
    char lock_file[128];

    debug(DBG_FUNCS, "my_unlock: sof");
    gen_lock_file_name_fnc(lock_file, lockwhat);
    unlink(lock_file);
    dontdelonexit(lock_file);
}

char *strncpy2(
char *to,
char *from,
int len)
{
    int i;

    for (i=0; i<len && i<strlen(from); i++)
        *(to+i)=*(from+i);
    *(to+i) = '\0';
    return(to);
}

char *getart(
char *str)
{
   return((str[0] >= 'A' && str[0] <= 'Z') ? "" : 
          (str[0] == 'a' || str[0] == 'e' || str[0] == 'i' || str[0] == 'o' ||
           str[0] == 'u')                  ? "an " : "a ");
}

void showdelonexitlist()
{
    struct delonexitrec *dp;
    char tmpbuf[1024];

    for (dp=delonexitlist, tmpbuf[0]='\0'; dp!=NULL; dp=dp->next)
        sprintf(tmpbuf+strlen(tmpbuf), "%s ", dp->name);
    debug(DBG_SPECIAL1, tmpbuf);
}

void delonexit(
char *filename)
{
    struct delonexitrec **dpaddr;

    for (dpaddr=&delonexitlist; *dpaddr!=NULL; dpaddr=&(*dpaddr)->next)
        ;

    if ((*dpaddr=calloc(1, sizeof(struct delonexitrec))) == NULL) 
        error("failed to allocate memory");

    (*dpaddr)->name = strdup(filename);
    (*dpaddr)->next = NULL;
}

void dontdelonexit(
char *filename)
{
    struct delonexitrec **dpaddr, *tmpp;
    
    for (dpaddr=&delonexitlist; *dpaddr!=NULL; dpaddr=&(*dpaddr)->next)
        if (strcmp(filename, (*dpaddr)->name) == 0) {
            tmpp = *dpaddr;
            *dpaddr = (*dpaddr)->next;
            free(tmpp->name);
            free(tmpp);
            return;
        }
}

void exitdel()
{
    while (delonexitlist != NULL) {
        unlink(delonexitlist->name);
        dontdelonexit(delonexitlist->name);
    }
}

void genericsighandler(
int signo)
{
    info("clearing up ...");
    exitdel();
    exit(EXIT_FAILURE);
}

==> animal-2.2.0.5.tar.gz: animal-2.2.0.5/bin/utils.c <==
/*
 *  Urgent bits first
 */

static char *utils_c_rcs_id = "$Header: /home/ahuxley/dev/supported/support/lib/RCS/utils.c,v 1.14 1999/06/07 09:05:40 ahuxley Exp $";

/*
 *  System includes - with reasons
 */

#include <stdio.h>				/* for NULL and *printf*() */
#include <stdarg.h>				/* for va_*() */
#include <string.h>				/* for strrchr(), strerror() */
#include <sys/types.h>                          /* for umask() */
#include <sys/stat.h>                   	/* for stat(), umask() */
#include <unistd.h>                    		/* for stat() */
#include <errno.h>				/* for errno */
#include <stdlib.h>				/* for exit() */
#ifndef MSDOS
#include <syslog.h>				/* for syslog(), LOG_*, etc */
#endif

/*
 *  Local includes with reasons			
 */

#include "utils.h" 				/* for pp symbols */

/*
 *  Private structure definitions
 */

struct delonexitrec {
    char *name;
    struct delonexitrec *next;
};

/*
 *  Private variables 
 */

static int state_syslog_opened = FALSE;
static struct delonexitrec *delonexitlist = NULL;

/*
 *  Global variables belonging to this file
 */

char *progname;
int i_am_a_daemon = FALSE;
unsigned long debug_level = DBG_WARNING|DBG_ERROR;

/*
 *  Functions start here
 */

char *mybasename(
char *cmdcp,
char *extnp)
{
    char *bncp;
    static char basenamebuf[64];

    bncp = ((bncp=strrchr(cmdcp, '/')) == NULL) ? cmdcp : bncp+1;

    if (extnp == NULL || *extnp == '\0' || strcmp(bncp+strlen(bncp)-strlen(extnp), extnp))
        return(bncp);
    else {
        strncpy2(basenamebuf, bncp, strlen(bncp)-strlen(extnp));
        return(basenamebuf);
    }
}

void internal(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    va_start(args, fmt);
    sprintf(msgbuf, "INTERNAL ERROR: ");
    vsprintf(msgbuf+strlen(msgbuf), fmt, args);
    va_end(args);

    if (!i_am_a_daemon)
        fprintf(stderr, "%s: %s\n", progname, msgbuf);
    else {
#ifndef MSDOS
        if (!state_syslog_opened) {
            openlog(progname, LOG_CONS|LOG_PID, FACILITY);
            state_syslog_opened = TRUE;
        }
        syslog(LOG_EMERG, msgbuf);
#else
        fprintf(stderr, "%s: %s\n", progname, msgbuf);
#endif
    }
    exit(255);
}

void error(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_ERROR) {
        va_start(args, fmt);
        sprintf(msgbuf, "ERROR: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_ERR, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
    exit(EXIT_FAILURE);
}

void warning(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_WARNING) {
        va_start(args, fmt);
        sprintf(msgbuf, "WARNING: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);
    
        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_WARNING, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

void info(
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (debug_level & DBG_INFO) {
        va_start(args, fmt);
        sprintf(msgbuf, "INFO: ");
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_INFO, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

void debug(
int level,
char *fmt,
...)
{
    va_list args;
    char msgbuf[512];

    if (level & debug_level) {
        va_start(args, fmt);
        sprintf(msgbuf, "DEBUG[%04x]: ", level);
        vsprintf(msgbuf+strlen(msgbuf), fmt, args);
        va_end(args);

        if (!i_am_a_daemon)
            fprintf(stderr, "%s: %s\n", progname, msgbuf);
        else {
#ifndef MSDOS
            if (!state_syslog_opened) {
                openlog(progname, LOG_CONS|LOG_PID, LOG_LOCAL0);
                state_syslog_opened = TRUE;
            }
            syslog(LOG_DEBUG, msgbuf);
#else
	    internal("msdos version has no syslog support");
#endif
        }
    }
}

/*
 *  A string copy function. It can cope with 'left-shift' overlaps, but
 *  not 'right-shifts'.
 */

char *strmove(
char *dst,
char *src)
{
    while (*dst++ = *src++)
        ;
}

pid_t my_lock(
char *lockwhat,
int (*gen_lock_file_name_fnc)(char *, char *))
{
    char lock_file[128];
    char tmp_lock_file[128];
    pid_t locking_pid;
    FILE *fp;
    struct stat statbuf;
    char procbuf[16];
    mode_t old_umask;

    debug(DBG_FUNCS, "my_lock: sof");

    gen_lock_file_name_fnc(lock_file, lockwhat);
    debug(DBG_LOCKS, "my_lock: lockfile=%s", lock_file);
    sprintf(tmp_lock_file, "%s.%d", lock_file, getpid());
    debug(DBG_LOCKS, "my_lock: tmplockfile=%s", tmp_lock_file);

    /* create temporary lock_file */
    debug(DBG_LOCKS, "my_lock: creating temporary lock");
    umask((old_umask = umask(0)) & 0022);
    delonexit(tmp_lock_file);
    if ((fp=fopen(tmp_lock_file, "w")) == (FILE *) NULL) {
	umask(old_umask);
        debug(DBG_LOCKS, "my_lock: temporary lock creation failed (errno=%d)", errno);
        dontdelonexit(tmp_lock_file);
        return(-1); 
    }
    umask(old_umask);
    fprintf(fp, "%d\n", getpid());
    fclose(fp);

    /* slide lock into place */
    debug(DBG_LOCKS, "my_lock: hard-linking temporary lock to real lock");
    delonexit(lock_file);
    if (link(tmp_lock_file, lock_file) == 0) {
        debug(DBG_LOCKS, "my_lock: hard-linking succeeded");
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(0);
    }
    dontdelonexit(lock_file);
    debug(DBG_LOCKS, "my_lock: hard-linking failed");

    /* if blocked examine the existing lock */
    debug(DBG_LOCKS, "my_lock: examining existing lock");
    if ((fp=fopen(lock_file, "r")) == (FILE *) NULL) {
        debug(DBG_LOCKS, "my_lock: lock file open for reading failed");
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(-1);
    }
    debug(DBG_LOCKS, "my_lock: lock file read failed");
    if (fscanf(fp, "%d", (int *) &locking_pid) != 1) {
        fclose(fp);
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(-1);
    }
    fclose(fp);

    debug(DBG_LOCKS, "my_lock: found a pid in lock file, checking if running");
    sprintf(procbuf, "/proc/%d", (int) locking_pid);
    if (stat(procbuf, &statbuf) == 0) {
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(locking_pid);
    }

    info("stale lock %s removed (pid=%d)", lock_file, (int) locking_pid);
    unlink(lock_file);

    /* slide lock into place */
    debug(DBG_LOCKS, "my_lock: hard-linking temporary lock to real lock (2nd attmept");
    delonexit(lock_file);
    if (link(tmp_lock_file, lock_file) == 0) {
        debug(DBG_LOCKS, "my_lock: hard-linking succeeded");
        unlink(tmp_lock_file);
        dontdelonexit(tmp_lock_file);
        return(0);
    }

    unlink(tmp_lock_file);
    dontdelonexit(tmp_lock_file);
    return(-1);
}

void my_unlock(
char *lockwhat,
int (*gen_lock_file_name_fnc)(char *, char *))
{
    char lock_file[128];

    debug(DBG_FUNCS, "my_unlock: sof");
    gen_lock_file_name_fnc(lock_file, lockwhat);
    unlink(lock_file);
    dontdelonexit(lock_file);
}

char *strncpy2(
char *to,
char *from,
int len)
{
    int i;

    for (i=0; i<len && i<strlen(from); i++)
        *(to+i)=*(from+i);
    *(to+i) = '\0';
    return(to);
}

char *getart(
char *str)
{
   return((str[0] >= 'A' && str[0] <= 'Z') ? "" : 
          (str[0] == 'a' || str[0] == 'e' || str[0] == 'i' || str[0] == 'o' ||
           str[0] == 'u')                  ? "an " : "a ");
}

void showdelonexitlist()
{
    struct delonexitrec *dp;
    char tmpbuf[1024];

    for (dp=delonexitlist, tmpbuf[0]='\0'; dp!=NULL; dp=dp->next)
        sprintf(tmpbuf+strlen(tmpbuf), "%s ", dp->name);
    debug(DBG_SPECIAL1, tmpbuf);
}

void delonexit(
char *filename)
{
    struct delonexitrec **dpaddr;

    debug(DBG_DELONEXIT, "delonexit: %s", filename);

    for (dpaddr=&delonexitlist; *dpaddr!=NULL; dpaddr=&(*dpaddr)->next)
        ;

    if ((*dpaddr=calloc(1, sizeof(struct delonexitrec))) == NULL) 
        error("failed to allocate memory");

    (*dpaddr)->name = strdup(filename);
    (*dpaddr)->next = NULL;
}

void dontdelonexit(
char *filename)
{
    struct delonexitrec **dpaddr, *tmpp;
    
    debug(DBG_DELONEXIT, "dontdelonexit: %s", filename);

    for (dpaddr=&delonexitlist; *dpaddr!=NULL; dpaddr=&(*dpaddr)->next)
        if (strcmp(filename, (*dpaddr)->name) == 0) {
            tmpp = *dpaddr;
            *dpaddr = (*dpaddr)->next;
            free(tmpp->name);
            free(tmpp);
            return;
        }
}

void exitdel()
{
    debug(DBG_DELONEXIT, "exitdel: removing all files");

    while (delonexitlist != NULL) {
        unlink(delonexitlist->name);
        dontdelonexit(delonexitlist->name);
    }
}

void genericsighandler(
int signo)
{
    info("cleaning up and exiting on signal %d", signo);
    exitdel();
    exit(EXIT_FAILURE);
}
